package com.sample;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.ListIterator;

/**
 * This class is a general purpose list with a name.  Its intended use
 * is to represent a fact in a rules based component.
 * 
 * @author crw14
 *
 * @param <T> type of element
 */
public class NamedList<T> extends ArrayList<T>
{
   private static final long serialVersionUID = 1L;
   
   private String name;
   
   /**
    * Create a list with the given parameters.
    * @param name name of list
    * @param elements elements to copy
    */
   public NamedList(String name, Collection<T> elements)
   {
      super(elements);
      this.name = name;
   }
   
   /**
    * Create a list with the given parameters.
    * @param name name of list
    * @param elements elements to copy
    */
   public NamedList(String name, T... elements)
   {
      this.name = name;
      for (T s: elements)
      {
         this.add(s);
      }
   }
   
   /**
    * Create a list with the given parameters.
    * @param name name of list
    */
   public NamedList(String name)
   {
      this(name, new ArrayList<T>());
   }
   
   /**
    * Get the name of this list.
    * @return name
    */
   public String getName()
   {
      return this.name;
   }
   
   /* (non-Javadoc)
    * @see java.util.AbstractList#iterator()
    */
   @Override
   public Iterator<T> iterator()
   {
      return this.namedListIterator();
   }
   
   /* (non-Javadoc)
    * @see java.util.AbstractList#listIterator()
    */
   @Override
   public ListIterator<T> listIterator()
   {
      return this.namedListIterator();
   }
   
   /**
    * Get an iterator to this list.  The iterator
    * will specifically be of type NamedListIterator whose
    * name will be the same as the name of the list.
    * @return the iterator
    */
   public NamedListIterator<T> namedListIterator()
   {
      final ListIterator<T> it = super.listIterator();
      return new NamedListIterator<T>(this, it);
   }
   
   /**
    * Property to obtain this list.  The purpose of this property
    * is to aid in the use of this list in a rules engine.
    * @return this list
    */
   public NamedList<T> getSelf()
   {
      return this;
   }
 
   @Override
   public String toString()
   {
      return "NamedList(" + this.name + "): " + super.toString();
   }
}
