package com.sample;

import java.util.ListIterator;

/**
 * This iterator accompanies the class NamedList, and contains a name.
 * Its purpose is to be a fact in a rules based component.
 * 
 * @author crw14
 *
 * @param <T> element type
 */
public class NamedListIterator<T> implements ListIterator<T>
{
   private NamedList<T> list;
   private ListIterator<T> it;

   /**
    * Create an iterator for the list given, delegating to the iterator given.
    * @param list list to iterate
    * @param it iterator to delegate to
    */
   public NamedListIterator(NamedList<T> list, ListIterator<T> it)
   {
      this.list = list;
      this.it = it;
   }
   
   protected NamedListIterator()
   {
      
   }

   /* (non-Javadoc)
    * @see java.util.ListIterator#add(java.lang.Object)
    */
   public void add(T o)
   {
      it.add(o);
   }

   /* (non-Javadoc)
    * @see java.util.ListIterator#hasNext()
    */
   public boolean hasNext()
   {
      return it.hasNext();
   }

   /* (non-Javadoc)
    * @see java.util.ListIterator#hasPrevious()
    */
   public boolean hasPrevious()
   {
      return it.hasPrevious();
   }

   /* (non-Javadoc)
    * @see java.util.ListIterator#next()
    */
   public T next()
   {
      return it.next();
   }

   /* (non-Javadoc)
    * @see java.util.ListIterator#nextIndex()
    */
   public int nextIndex()
   {
      return it.nextIndex();
   }

   /* (non-Javadoc)
    * @see java.util.ListIterator#previous()
    */
   public T previous()
   {
      return it.previous();
   }

   /* (non-Javadoc)
    * @see java.util.ListIterator#previousIndex()
    */
   public int previousIndex()
   {
      return it.previousIndex();
   }

   /* (non-Javadoc)
    * @see java.util.ListIterator#remove()
    */
   public void remove()
   {
      it.remove();
   }

   /* (non-Javadoc)
    * @see java.util.ListIterator#set(java.lang.Object)
    */
   public void set(T o)
   {
      it.set(o);
   }

   /**
    * Get the name of this iterator.  This name is the same as the name of the
    * list it iterates over.
    * @return name
    */
   public String getName()
   {
      return this.list.getName();
   }
   
   /**
    * Reset this iterator to the first element.
    */
   public void resetToFirst()
   {
      while (this.hasPrevious())
      {
         this.previous();
      }
   }
   
   /**
    * Reset this iterator to the last element.
    */
   public void resetToLast()
   {
      while (this.hasNext())
      {
         this.next();
      }
   }
}